@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.3
+////  Version 1.4
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -63,7 +63,7 @@
 #ifndef RSFREGR_H
 #define RSFREGR_H
 #include "node.h"
-void getMeanResponse(uint mode, uint treeID);
+void getMeanResponse(uint treeID);
 void updateEnsembleMean(uint    mode, 
                         uint    treeID, 
                         double *ensemblePredictor);
